"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseSignUrl = exports.signTxSchema = exports.signBaseSchema = exports.arrayOrString = void 0;
const omit_1 = __importDefault(require("lodash/omit"));
const yup_1 = require("yup");
const transaction_1 = require("./transaction");
const validUrlSchema_1 = require("./validUrlSchema");
const helpers_1 = require("../../helpers");
const types_1 = require("../../types");
const sdkDappCore_1 = require("../../lib/sdkDappCore");
const maxTransactions = window.opener ? 50 : 5;
const validString = (0, yup_1.mixed)().test('string', 'Must be string', (text) => typeof text === 'string');
const validArray = (0, yup_1.array)()
    .of((0, yup_1.string)())
    .required()
    .min(1, 'At least one value')
    .max(maxTransactions, `Maximum ${maxTransactions} values`);
exports.arrayOrString = (0, yup_1.lazy)((value) => typeof value === 'string' ? validString : validArray);
const nullableArrayOrString = (0, yup_1.lazy)((value) => {
    switch (true) {
        case value === undefined:
            return (0, yup_1.mixed)().notRequired();
        case typeof value === 'string':
            return validString;
        default:
            return validArray;
    }
});
exports.signBaseSchema = (0, yup_1.object)({
    data: nullableArrayOrString,
    token: nullableArrayOrString,
    chainId: nullableArrayOrString,
    sender: nullableArrayOrString,
    senderUsername: nullableArrayOrString,
    receiverUsername: nullableArrayOrString,
    gasLimit: exports.arrayOrString,
    guardian: nullableArrayOrString,
    guardianSignature: nullableArrayOrString,
    gasPrice: exports.arrayOrString,
    receiver: (0, yup_1.mixed)().test('sameNumberOfValues', 'All transactions must have equal receiver field values', function (value) {
        const { data } = this.parent;
        if (typeof data === 'string' && (0, helpers_1.isNftOrMultiEsdtTx)(data)) {
            return true;
        }
        try {
            exports.arrayOrString.validateSync(value);
            return true;
        }
        catch {
            return false;
        }
    }),
    value: (0, yup_1.mixed)().test('sameNumberOfValues', 'All transactions must have equal value field values', function (value) {
        const { data } = this.parent;
        if (typeof data === 'string' && (0, helpers_1.isNftOrMultiEsdtTx)(data)) {
            return true;
        }
        try {
            exports.arrayOrString.validateSync(value);
            return true;
        }
        catch {
            return false;
        }
    }),
    nonce: exports.arrayOrString,
    callbackUrl: validUrlSchema_1.validUrlSchema
        .required()
        .test('sameNumberOfValues', 'All transactions must have equal array values', function () {
        const fields = (0, omit_1.default)(this.parent, ['modal', 'callbackUrl']);
        const allValues = Object.values(fields);
        const allArrays = allValues.every((value) => Array.isArray(value));
        if (allArrays) {
            const maxLengh = Math.max(...allValues.map((arr) => arr.length));
            return allValues.every((arr) => arr.length === maxLengh);
        }
        return true;
    })
        .test('differentDataTypes', 'Values must be either strings or arrays', function () {
        const fields = (0, omit_1.default)(this.parent, ['modal', 'callbackUrl']);
        const allValues = Object.values(fields);
        const someArrays = allValues.some((value) => Array.isArray(value));
        const someStrings = allValues.some((value) => typeof value === 'string');
        return !(someArrays && someStrings);
    }),
    [types_1.HookSearchParamsEnum.executeAfterSign]: (0, yup_1.string)(),
    // TODO: maybe remove?
    postMessage: (0, yup_1.string)().test('mustBeTrue', 'If set, postMessage param must be true', function (postMessage) {
        if (postMessage) {
            return postMessage === 'true';
        }
        return true;
    })
}).required();
const signTxSchema = ({ isMainnet, hookWhitelist, chainId, isSignHook }) => {
    const txFields = (0, transaction_1.transactionFields)({
        isMainnet,
        hookWhitelist,
        isSignHook
    });
    return (0, yup_1.object)({
        ...txFields,
        nonce: (0, yup_1.string)()
            .required()
            .test('validInteger', 'Nonce invaild number', (value) => {
            if (value != null) {
                return (0, sdkDappCore_1.stringIsInteger)(value);
            }
            return true;
        }),
        chainID: (0, yup_1.string)().test('sameAsConfig', 'Invalid Chain Id', (value) => {
            if (value != null && value !== '') {
                return value === chainId;
            }
            return true;
        })
    }).required();
};
exports.signTxSchema = signTxSchema;
const parseSignUrl = (search) => {
    const hook = (0, helpers_1.parseQueryParams)(search);
    const { callbackUrl, executeAfterSign, ...fields } = hook;
    const containsArrays = Object.values(fields).some((entry) => Array.isArray(entry));
    if (containsArrays) {
        const transactions = [];
        Object.keys(fields).forEach((key) => {
            const values = fields[key];
            if (Array.isArray(values)) {
                values.forEach((value, i) => {
                    const transaction = transactions[i];
                    transactions[i] = {
                        ...(transaction ? { ...transaction } : {}),
                        [key]: `${value}`
                    };
                });
            }
        });
        const parsedUrl = {
            callbackUrl: callbackUrl || '',
            txs: transactions
        };
        if (executeAfterSign) {
            parsedUrl.executeAfterSign = executeAfterSign;
        }
        return parsedUrl;
    }
    else {
        const txs = (Object.keys(fields).length > 0 ? [{ ...fields }] : []);
        const parsedUrl = {
            callbackUrl: callbackUrl || '',
            txs
        };
        if (executeAfterSign) {
            parsedUrl.executeAfterSign = executeAfterSign;
        }
        return parsedUrl;
    }
};
exports.parseSignUrl = parseSignUrl;
//# sourceMappingURL=sign.js.map